/* DPMI.H:  DPMI constants, structs and prototypes
*/

#if !defined (DPMI_H)
#define DPMI_H

#if !defined (BOOL)
enum BOOL { FALSE, TRUE };
#endif

// Access Rights Word defines
#define PRESENTBIT               0x80
#define DPLBITS                  0x60
#define SEGMENTBIT               0x10
#define TYPEBITS                 0x0E
#define ACCESSEDBIT              0x01

#define DPMINT                   0x31

#define DPMI_SETDESCRIPACCESS    0x0009
#define DPMI_CREATEALIASDESCRIP  0x000A
#define DPMI_ALLOCREALSEG        0x0100
#define DPMI_FREEREALSEG         0x0101
#define DPMI_SIMULATEREALMODEINT 0x0300
#define DPMI_GETVERSION          0x0400
#define DPMI_GETFREEMEMORYINFO   0x0500
#define DPMI_GETMEMORYINFO       0x050B
#define DPMI_GETPAGESIZE         0x0604

#define DPMI_SUCCESS             0

// Memory control block is damaged.
#define DPMI_MCBDAMAGED          7

// Insufficient memory to complete operation.
#define DPMI_INSUFFICIENTMEMORY  8

// Incorrect memory segment specified
#define DPMI_INVALIDMEMORYSEG    9

// Returned in response to any function call which is not
// implemented by this host, because the requested function is
// either undefined or optional.
#define DPMI_UNSUPPORTEDFUNCTION 0x8001

// Some object is in the wrong state for the requested
// operation.
#define DPMI_INVALIDSTATE        0x8002

// The requested operation would endanger system integrity,
// e.g., a request to map linear addresses onto system code
// or data.
#define DPMI_SYSINTEGRITY        0x8003

// Host detected a deadlock situation.
#define DPMI_DEADLOCKHOST        0x8004

// A pending serialization request was cancelled.
#define DPMI_REQUESTCANCEL       0x8005

// The DPMI host cannot allocate internal resources to
// complete an operation.
#define DPMI_NOAVAILRESOURCE     0x8010

// Host is unable to allocate a descriptor
#define DPMI_NOAVAILDESCRIPTOR   0x8011

// Host is unable to allocate the required linear memory
#define DPMI_NOAVAILLINEARMEM    0x8012

// Host is unable to allocate the required physical memory
#define DPMI_NOAVAILPHYSICALMEM  0x8013

// Host is unable to allocate the required backing store
#define DPMI_NOAVAILBACKINGSTOR  0x8014

// Host is unable to allocate the required callback address
#define DPMI_NOAVAILCALLBACK     0x8015

// Host is unable to allocate the required handle
#define DPMI_NOAVAILHANDLE       0x8016

// A locking operation exceeds the maximum count maintained
// by the host.
#define DPMI_LOCKCOUNTEXCEEDED   0x8017

// Resource owned exclusively A request for serialization of
// a shared memory block could not be satisfied because it
// is already serialized exclusively by another client.
#define DPMI_EXCLUSIVEOWNEDRES   0x8018

// Resource owned shared A request for exclusive serialization
// of a shared memory block could not be satisfied because it
// is already serialized shared by another client.
#define DPMI_SHAREDOWNEDRES      0x8019

// A numeric or flag parameter has an invalid value
#define DPMI_INVALIDVALUE        0x8021

// A selector does not correspond to a valid descriptor.
#define DPMI_INVALIDSELECTOR     0x8022

// A handle parameter is invalid.
#define DPMI_INVALIDHANDLE       0x8023

// A callback parameter is invalid.
#define DPMI_INVALIDCALLBACK     0x8024

// A linear address range (either supplied as a parameter or
// implied bythe call) is invalid.
#define INVALIDLINEARADDRESS     0x8025

// The request is not supported by the underlying hardware.
#define DPMI_INVALIDREQUEST      0x8026


/////////////////////////////////////////// DPMI_ERROR /////////
typedef struct {
  unsigned long error_code;
  char          *error_msg;
} DPMI_ERROR, *PDPMI_ERROR;

/////////////////////////////////////////// RREGS //////////////
typedef struct {
  unsigned long	di, si, bp;
  unsigned long rsvd;
  unsigned long bx, dx, cx, ax;

  unsigned int flags, es, ds;
  unsigned int fs, gs;
  unsigned int ip, cs;
  unsigned int sp, ss;
} RREGS, *PRREGS;

/////////////////////////////////////////// MEMINFO ////////////
// .9 host memory info structure
typedef struct {
  unsigned long largest_block;
  unsigned long max_unlocked;
  unsigned long max_locked_page;
  unsigned long linear_memory_pages;
  unsigned long total_unlocked_pages;
  unsigned long total_free_pages;
  unsigned long total_physical_pages;
  unsigned long free_linear_pages;
  unsigned long partition_pages;
  unsigned char rsvd[12];  // should be 12 0xff bytes
} MEMINFO, *PMEMINFO;

/////////////////////////////////////////// MEMINFO1 ///////////
typedef struct {
  unsigned int  total_allocated_memory;
  unsigned int  total_allocated_virtual_memory;
  unsigned int  total_available_virtual_memory;
  unsigned int  total_allocated_virtual_virtual_memory;
  unsigned int  total_available_virtual_virtual_memory;
  unsigned int  total_allocated_virtual_client_memory;
  unsigned int  total_available_virtual_client_memory;
  unsigned int  total_locked_client_bytes;
  unsigned int  maximum_locked_client_bytes;
  unsigned int  highest_linear_client_address;
  unsigned int  largest_available_free_memory_block;
  unsigned int  minium_allocation_unit;
  unsigned int  allocation_alignment_unit;
  unsigned char rsvd[76];
} MEMINFO1, PMEMINFO1;


void fail(const char *errMsg, unsigned short  code);


#endif // DPMI_H


